import 'package:flutter/material.dart';
import 'package:millui/millui.dart';
import 'dart:math';

import '../../app_provider.dart';

List<Color> colors = [
  Colors.blue[300],
  Colors.indigo[300],
  Colors.greenAccent,
  Colors.orange[300],
  Colors.yellow[300],
  Colors.pink[200],
  Colors.purple[200],
  Colors.brown[200],
];

class Refash extends StatefulWidget {
  _Refash createState() => new _Refash();
}

class _Refash extends State<Refash> {
  int i = 0;
  List _items = List.generate(10, (index) => 'item$index');
  bool _visibileSel = false;
  Future<dynamic> _refresh() async {
    return await Future.delayed(Duration(milliseconds: 1000), () {
      if (mounted) {
        setState(() {
          _items = List.generate(10, (index) => 'item$index');
        });
      }
      return _items;
    });
  }

  Future<dynamic> _onLoad() async {
    await Future.delayed(Duration(milliseconds: 500), () {
      i++;
      List l = List.generate(10, (index) => 'item${index + i * 10}');
      _items.addAll(l);
      if (mounted) {
        setState(() {
          _items = _items;
        });
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    bool isCn = getLanage(context);

    return Scaffold(
      appBar: AppBar(
        title: Text('refresh'),
      ),
      body: Stack(
        children: [
          Stack(
            children: [
              Column(
                children: [
                  Container(
                    alignment: Alignment.center,
                    color: Colors.cyan,
                    height: 60,
                    child: Text(
                      'Top',
                      style: TextStyle(
                        color: Colors.white,
                      ),
                    ),
                  ),
                  Expanded(
                    flex: 1,
                    child: MillRefresh(
                      cacheExtent: 1000.0,
                      timeout: 6,
                      sliverList: _buildList(_items),
                      dropText: isCn ? '下拉刷新' : 'pulldown ↓',
                      releaseText: isCn ? '释放加载' : 'release ↑',
                      footReleaseText: isCn ? '释放加载' : 'release ↓',
                      noDataText: isCn ? '没有更多数据' : 'NO DATA',
                      refreshing: MillBarBeatLoading(),
                      loading: MillBallRotateLoading(
                        size: 2,
                        paintSize: 16,
                      ),
                      initData: _refresh,
                      onRefresh: _refresh,
                      onLoad: _onLoad,
                      onNotification: (notification) {
                        // if (notification is ScrollUpdateNotification) {
                        //   print('update');
                        // } else if (notification is ScrollEndNotification) {
                        //   print('end');
                        // }
                        return true;
                      },
                      preAdapter: [
                        SliverToBoxAdapter(
                          child: Container(
                            alignment: Alignment.center,
                            height: 60,
                            child: Text(isCn ? '你好, 一个可滚动组件' : 'hellow'),
                          ),
                        ),
                        SliverPersistentHeader(
                          pinned: true, //是否固定在顶部
                          floating: true,
                          delegate: _SliverAppBarDelegate(
                            minHeight: 60, //收起的高度
                            maxHeight: 60, //展开的最大高度
                            child: Container(
                              height: 60,
                              alignment: Alignment.center,
                              child: Text(
                                isCn ? '吸顶导航' : 'fix',
                                style: TextStyle(
                                  color: Colors.white,
                                ),
                              ),
                              color: Colors.redAccent,
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
              Positioned(
                  left: 0,
                  top: 60,
                  child: Offstage(
                      offstage: !_visibileSel,
                      child: GestureDetector(
                          onTap: () {
                            setState(() {
                              _visibileSel = false;
                            });
                          },
                          child: Opacity(
                            opacity: 0.6,
                            child: Container(
                              alignment: Alignment(0, 0),
                              width: double.maxFinite,
                              height: double.maxFinite,
                              decoration:
                                  BoxDecoration(color: Color(0xff000000)),
                              child: Text('s'),
                            ),
                          )))),
            ],
          )
        ],
      ),
    );
  }
}

Widget _buildList(items) {
  return SliverList(
      delegate: SliverChildBuilderDelegate(
    (context, index) {
      return Container(
        color: colors[index % 8],
        child: Card(data: '${items[index]}'),
      );
    },
    childCount: items.length,
  ));
}

class Card extends StatefulWidget {
  Card({
    this.data,
  });
  final data;
  _Card createState() => _Card();
}

class _Card extends State<Card> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.all(20),
      child: Text(
        '${widget.data}',
        style: TextStyle(
          color: Colors.white,
        ),
      ),
    );
  }
}

class _SliverAppBarDelegate extends SliverPersistentHeaderDelegate {
  _SliverAppBarDelegate({
    @required this.minHeight,
    @required this.maxHeight,
    @required this.child,
  });

  final double minHeight;
  final double maxHeight;
  final Widget child;

  @override
  double get minExtent => minHeight;

  @override
  double get maxExtent => max(maxHeight, minHeight);

  @override
  Widget build(
      BuildContext context, double shrinkOffset, bool overlapsContent) {
    return new SizedBox.expand(child: child);
  }

  @override
  bool shouldRebuild(_SliverAppBarDelegate oldDelegate) {
    return maxHeight != oldDelegate.maxHeight ||
        minHeight != oldDelegate.minHeight ||
        child != oldDelegate.child;
  }
}
