import 'package:flutter/material.dart';
import 'picker_widget.dart';

class MillDatePicker extends StatefulWidget {
  MillDatePicker({
    this.minYear,
    this.maxYear,
    this.defaultYear,
    this.defaultMonth,
    this.defaultDay,
    this.onChanged,
  });
  final int minYear;
  final int maxYear;
  final int defaultYear;
  final int defaultMonth;
  final int defaultDay;
  final Function onChanged;

  @override
  State<MillDatePicker> createState() => _DatePicker();
}

class _DatePicker extends State<MillDatePicker> {
  List years = [];
  List months = [];
  int _year;
  int _month;
  int _day;

  List mmonth = [1, 3, 5, 7, 8, 10, 12];
  int get maxDay {
    if (_year != null &&
        (_year % 400 == 0 || _year % 100 != 0 && _year % 4 == 0) &&
        _month == 2) {
      return 29;
    }
    if (_month != null) {
      int index = mmonth.indexWhere((el) => el == _month);
      if (index >= 0) {
        return 31;
      }
    }
    if (_month == 2) {
      return 28;
    }
    return 30;
  }

  _change() {
    if (widget.onChanged != null) {
      widget.onChanged('$_year-$_month-$_day');
    }
  }

  @override
  initState() {
    super.initState();
    for (int i = widget.minYear; i < widget.maxYear; i++) {
      years.add({
        'label': '${i}',
        'value': i,
      });
    }
    for (int i = 1; i < 13; i++) {
      months.add({
        'label': '${i}',
        'value': i,
      });
    }

    _year = widget.defaultYear ?? widget.minYear;
    _month = widget.defaultMonth ?? 1;
    _day = widget.defaultDay ?? 1;
  }

  @override
  Widget build(BuildContext context) {
    List days = [];
    for (int i = 1; i < maxDay + 1; i++) {
      days.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultYearIndex =
        years.indexWhere((el) => el['value'] == widget.defaultYear);
    int defaultMonthIndex =
        months.indexWhere((el) => el['value'] == widget.defaultMonth);
    int defaultDayIndex =
        days.indexWhere((el) => el['value'] == widget.defaultDay);

    return Row(
      children: [
        Expanded(
          child: PickerWidget(
            key: ValueKey('year'),
            list: years,
            defaultIndex: defaultYearIndex >= 0 ? defaultYearIndex : 0,
            onChanged: (index) {
              setState(() {
                _year = years[index]['value'];
              });
              _change();
            },
          ),
        ),
        Expanded(
          child: PickerWidget(
            key: ValueKey('month'),
            list: months,
            defaultIndex: defaultMonthIndex >= 0 ? defaultMonthIndex : 0,
            onChanged: (index) {
              setState(() {
                _month = months[index]['value'];
                _day = widget.defaultDay ?? 1;
              });
              _change();
            },
          ),
        ),
        Expanded(
          child: PickerWidget(
            key: ValueKey('day'),
            defaultIndex: defaultDayIndex >= 0 ? defaultDayIndex : 0,
            list: days,
            onChanged: (index) {
              _day = days[index]['value'];
              _change();
            },
          ),
        ),
      ],
    );
  }
}

showMillDatePicker({
  context,
  key,
  Color backgroundColor = const Color(0xffffffff),
  int minYear = 1900,
  int maxYear = 2100,
  int defaultYear,
  int defaultMonth,
  int defaultDay,
  String title,
  String cancelText,
  String confirmText,
  Function onCancel,
  Function onConfirm,
  TextStyle confirmTextStyle = const TextStyle(
    color: Color(0xff1578ff),
    fontSize: 15,
  ),
  TextStyle cancelTextStyle = const TextStyle(
    color: Color(0xff1578ff),
    fontSize: 15,
  ),
  TextStyle titleTextStyle = const TextStyle(
    color: Color(0xff343434),
    fontSize: 18,
  ),
}) {
  Radius r = Radius.circular(20);
  String newdate = '';
  return showModalBottomSheet(
    isScrollControlled: true,
    context: context,
    shape: RoundedRectangleBorder(
      borderRadius: BorderRadius.only(topLeft: r, topRight: r),
    ),
    clipBehavior: Clip.antiAlias,
    backgroundColor: backgroundColor,
    builder: (BuildContext context) {
      return FractionallySizedBox(
        widthFactor: 1,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            title != null
                ? Container(
                    height: 60.0,
                    alignment: Alignment.center,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        GestureDetector(
                          onTap: () {
                            if (onCancel != null) {
                              onCancel();
                            }
                            Navigator.pop(context);
                          },
                          child: Container(
                            color: Colors.transparent,
                            padding: EdgeInsets.symmetric(
                                horizontal: 20, vertical: 10),
                            child: Text(cancelText ?? '取消',
                                style: cancelTextStyle),
                          ),
                        ),
                        Text(
                          title,
                          style: titleTextStyle,
                        ),
                        GestureDetector(
                          onTap: () {
                            if (onConfirm != null) {
                              onConfirm(newdate);
                            }
                            Navigator.pop(context);
                          },
                          child: Container(
                            color: Colors.transparent,
                            padding: EdgeInsets.symmetric(
                                horizontal: 20, vertical: 10),
                            child: Text(confirmText ?? '确定',
                                style: confirmTextStyle),
                          ),
                        )
                      ],
                    ),
                  )
                : Container(),
            MillDatePicker(
              minYear: minYear,
              maxYear: maxYear,
              defaultYear: defaultYear,
              defaultMonth: defaultMonth,
              defaultDay: defaultDay,
              onChanged: (date) {
                newdate = date;
              },
            ),
          ],
        ),
      );
    },
  );
}
