import 'dart:math';

import 'package:flutter/material.dart';
import 'picker_widget.dart';

class MillPicker {
  static year({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    int minYear = 1900,
    int maxYear = 2100,
    initValue,
  }) {
    List list = [];
    for (int i = minYear; i < maxYear; i++) {
      list.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultIndex = 0;
    if (initValue != null) {
      defaultIndex = list.indexWhere((el) => el['value'] == initValue);
    }
    Radius r = Radius.circular(20);
    return showModalBottomSheet(
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return PickerWidget(
          key: key,
          defaultIndex: defaultIndex,
          list: list,
        );
      },
    );
  }

  static month({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    initValue,
  }) {
    List list = [];
    for (int i = 1; i < 13; i++) {
      list.add({
        'label': '${i}',
        'value': i,
      });
    }
    int defaultIndex = 0;
    if (initValue != null) {
      defaultIndex = list.indexWhere((el) => el['value'] == initValue);
    }
    if (defaultIndex < 0) {
      defaultIndex = 0;
    }
    Radius r = Radius.circular(20);
    return showModalBottomSheet(
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return PickerWidget(
          key: key,
          defaultIndex: defaultIndex,
          list: list,
        );
      },
    );
  }

  static pick({
    context,
    key,
    Color backgroundColor = const Color(0xffffffff),
    List<dynamic> initValue,
    List list,
    String title,
    String cancelText,
    String confirmText,
    Function onCancel,
    Function onConfirm,
    int account,
    String labelAlias = 'label',
    String valueAlias = 'value',
    TextStyle confirmTextStyle = const TextStyle(
      color: Color(0xff1578ff),
      fontSize: 15,
    ),
    TextStyle cancelTextStyle = const TextStyle(
      color: Color(0xff1578ff),
      fontSize: 15,
    ),
    TextStyle titleTextStyle = const TextStyle(
      color: Color(0xff343434),
      fontSize: 18,
    ),
  }) {
    List value = [];
    Radius r = Radius.circular(20);
    return showModalBottomSheet(
      isScrollControlled: true,
      context: context,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.only(topLeft: r, topRight: r),
      ),
      clipBehavior: Clip.antiAlias,
      backgroundColor: backgroundColor,
      builder: (BuildContext context) {
        return FractionallySizedBox(
          widthFactor: 1,
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              title != null
                  ? Container(
                      height: 60.0,
                      alignment: Alignment.center,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          GestureDetector(
                            onTap: () {
                              if (onCancel != null) {
                                onCancel();
                              }
                              Navigator.pop(context);
                            },
                            child: Container(
                              color: Colors.transparent,
                              padding: EdgeInsets.symmetric(
                                  horizontal: 20, vertical: 10),
                              child: Text(cancelText ?? '取消',
                                  style: cancelTextStyle),
                            ),
                          ),
                          Text(
                            title,
                            style: titleTextStyle,
                          ),
                          GestureDetector(
                            onTap: () {
                              if (onConfirm != null) {
                                if (value.length == 1) {
                                  onConfirm(value[0]);
                                } else {
                                  onConfirm(value);
                                }
                              }
                              Navigator.pop(context);
                            },
                            child: Container(
                              color: Colors.transparent,
                              padding: EdgeInsets.symmetric(
                                  horizontal: 20, vertical: 10),
                              child: Text(confirmText ?? '确定',
                                  style: confirmTextStyle),
                            ),
                          )
                        ],
                      ),
                    )
                  : Container(),
              MillMulPicker(
                key: key,
                account: account,
                initValue: initValue,
                list: list,
                labelAlias: labelAlias,
                valueAlias: valueAlias,
                onChanged: (val) {
                  value = val;
                },
              ),
            ],
          ),
        );
      },
    );
  }
}

class MillMulPicker extends StatefulWidget {
  MillMulPicker({
    this.key,
    @required this.list,
    this.account,
    this.initValue,
    this.labelAlias = 'label',
    this.valueAlias = 'value',
    @required this.onChanged,
  }) : super(key: key);
  final Key key;
  final List list;
  final int account;
  final List<dynamic> initValue;
  final String labelAlias;
  final String valueAlias;
  final onChanged;
  _MillMulPicker createState() => _MillMulPicker();
}

class _MillMulPicker extends State<MillMulPicker> {
  List<List> stacks = [];
  List<int> defaultIndex = [];
  List values = [];

  @override
  void initState() {
    List tree = widget.list;
    int max = widget.account ??
        (widget.initValue == null
            ? 1
            : (widget.initValue.length > 0 ? widget.initValue.length : 1));
    for (int i = 0; i < max; i++) {
      if (tree.length > 0 && max > stacks.length) {
        int index = tree.indexWhere((el) {
          if (widget.initValue != null && max == widget.initValue.length) {
            return el[widget.valueAlias] == widget.initValue[i];
          } else {
            return false;
          }
        });
        if (index < 0) {
          index = 0;
        }
        defaultIndex.add(index);
        stacks.add(tree);
        values.add({
          widget.labelAlias: tree[index][widget.labelAlias],
          widget.valueAlias: tree[index][widget.valueAlias],
        });
        tree = tree[index]['children'] ?? [];
      }
    }
    widget.onChanged(values);

    super.initState();
  }

  _change(int index, int scrollIndex) {
    Future.delayed(Duration(milliseconds: 180)).then((e) {
      Map obj = stacks[index][scrollIndex];
      var value = obj;
      values[index] = {
        widget.labelAlias: value[widget.labelAlias],
        widget.valueAlias: value[widget.valueAlias],
      };
      if (index + 1 != stacks.length) {
        for (int i = index + 1; i < stacks.length; i++) {
          if (obj != null) {
            stacks[i] = obj['children'] ?? [];
            if (stacks[i].length > 0) {
              obj = stacks[i][0];
            }
            values[i] = {
              widget.labelAlias: obj[widget.labelAlias] ?? '',
              widget.valueAlias: obj[widget.valueAlias] ?? '',
            };
          } else {
            stacks[i] = [];
          }
        }
      }

      setState(() {
        stacks = stacks;
      });
      if (values.length > 0) {
        widget.onChanged(values);
      } else {
        widget.onChanged([]);
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      children: List.generate(stacks.length, (index) {
        return Expanded(
          flex: 1,
          child: PickerWidget(
            key: ValueKey('$index'),
            labelAlias: widget.labelAlias,
            valueAlias: widget.valueAlias,
            defaultIndex: defaultIndex[index] ?? 0,
            list: stacks[index],
            onChanged: (value) {
              _change(index, value);
            },
          ),
        );
      }),
    );
  }
}
