import 'dart:ui';

import 'package:flutter/material.dart';
import './scroll_notification_listener.dart';
import './scroll_physics.dart';
import './slive_header.dart';
import './slive_footer.dart';

class MillRefresh extends StatefulWidget {
  MillRefresh({
    Key key,
    @required this.sliverList,
    this.onRefresh,
    this.onLoad,
    this.refreshFinished,
    this.refreshing,
    this.loading,
    this.headerBgColor = const Color(0xfff3f3f3),
    this.footerBgColor = const Color(0xfff3f3f3),
    this.preAdapter,
    this.sufAdapter,
    this.timeout = 10,
    this.error,
    this.hasMore = true,
    this.dropText = '下拉刷新',
    this.releaseText = '释放加载',
    this.footReleaseText = '释放加载',
    this.noDataText = '没有更多数据',
    this.onNotification,
    this.cacheExtent,
    this.controller,
    this.initData,
    this.initDataSuccess,
    this.initDataError,
  }) : super(key: key);

  final sliverList;
  final Widget refreshFinished;
  final Widget refreshing;
  final Widget loading;
  final Color headerBgColor;
  final Color footerBgColor;
  final Future<dynamic> Function() onRefresh;
  final Future<dynamic> Function() onLoad;
  final Future<dynamic> Function() initData;
  final Widget error;
  final timeout;
  final bool hasMore;
  final List<Widget> preAdapter;
  final SliverToBoxAdapter sufAdapter;
  final String dropText;
  final String releaseText;
  final String footReleaseText;
  final String noDataText;
  final bool Function(ScrollNotification) onNotification;
  final double cacheExtent;
  final ScrollController controller;
  final Widget initDataSuccess;
  final Widget initDataError;

  @override
  _MillRefresh createState() => new _MillRefresh();
}

class _MillRefresh extends State<MillRefresh>
    with SingleTickerProviderStateMixin {
  ValueNotifier<BouncingSettings> _bouncingNotifier;
  MillRefreshPhysics _physics;
  ValueNotifier<bool> _focusNotifier;
  ValueNotifier<double> _extraExtentNotifier;
  Animation _sizeAnimation;
  AnimationController _animationController;
  bool _initing = false;
  int _initState = 0;
  @override
  void initState() {
    super.initState();
    _animationController = AnimationController(
      duration: Duration(milliseconds: 400),
      vsync: this,
    );
    _sizeAnimation = Tween(begin: 1.0, end: 0.01).animate(
      CurvedAnimation(
        parent: _animationController,
        curve: Interval(0.5, 1.0),
      ),
    );
    _extraExtentNotifier = ValueNotifier<double>(0.0);
    _focusNotifier = ValueNotifier<bool>(false);
    _bouncingNotifier = ValueNotifier<BouncingSettings>(BouncingSettings());
    _createPhysics();

    if (widget.initData != null) {
      _initing = true;
      widget.initData()
        ..then((value) {
          setState(() {
            _initState = 1;
          });
        })
        ..catchError((err) {
          setState(() {
            _initState = 2;
          });
        })
        ..whenComplete(() => _closeInit());
    }
  }

  _closeInit() {
    Future.delayed(Duration(milliseconds: 800), () {
      _animationController.forward();
      Future.delayed(Duration(milliseconds: 404), () {
        setState(() {
          _initing = false;
        });
      });
    });
  }

  @override
  void setState(fn) {
    if (mounted) {
      super.setState(fn);
    }
  }

  void _createPhysics() {
    _bouncingNotifier.value = BouncingSettings(
      top: true,
      bottom: true,
    );
    _physics = MillRefreshPhysics(
      bouncingNotifier: _bouncingNotifier,
      extraExtentNotifier: _extraExtentNotifier,
    );
  }

  // 销毁
  void dispose() {
    _focusNotifier.dispose();
    _extraExtentNotifier.dispose();
    _animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> sliverList = [];

    if (widget.sliverList is SliverList ||
        widget.sliverList is SliverToBoxAdapter) {
      sliverList.add(widget.sliverList);
    } else if (widget.sliverList is List) {
      sliverList.addAll(widget.sliverList);
    }

    if (widget.preAdapter != null) {
      sliverList.insertAll(0, widget.preAdapter);
    }

    if (widget.sufAdapter != null) {
      sliverList.add(widget.sufAdapter);
    }

    if (widget.onRefresh != null) {
      sliverList.insert(
        0,
        new SliveHeader(
          timeout: widget.timeout,
          error: widget.error,
          finished: widget.refreshFinished,
          loading: widget.refreshing,
          onRefresh: _initing ? null : widget.onRefresh,
          focusNotifier: _focusNotifier,
          backgroundColor: widget.headerBgColor,
          dropText: widget.dropText,
          releaseText: widget.releaseText,
        ),
      );
    }

    if (_initing) {
      sliverList.insert(
        0,
        SliverToBoxAdapter(
          child: SizeTransition(
            axis: Axis.vertical,
            sizeFactor: _sizeAnimation,
            child: Container(
              alignment: Alignment.center,
              height: 60,
              child: _initState == 0
                  ? widget.refreshing
                  : _initState == 1
                      ? widget.initDataSuccess ??
                          Container(
                            child: Text(
                              '数据已更新',
                              style: TextStyle(
                                color: Color(0xff999999),
                                fontSize: 14,
                              ),
                            ),
                          )
                      : widget.initDataError ??
                          Container(
                            child: Text(
                              '数据更新失败',
                              style: TextStyle(
                                color: Color(0xfff56c6c),
                                fontSize: 12,
                              ),
                            ),
                          ),
            ),
          ),
        ),
      );
    }

    if (widget.onLoad != null) {
      sliverList.add(
        new SliveFooter(
          hasMore: widget.hasMore,
          loading: widget.loading,
          onLoad: widget.onLoad,
          focusNotifier: _focusNotifier,
          backgroundColor: widget.footerBgColor,
          extraExtentNotifier: _extraExtentNotifier,
          footReleaseText: widget.footReleaseText,
          noDataText: widget.noDataText,
        ),
      );
    }

    return ScrollNotificationListener(
      onFocus: (focus) {
        _focusNotifier.value = focus;
      },
      onNotification: widget.onNotification,
      child: ScrollConfiguration(
        behavior: ScrollConfiguration.of(context),
        child: CustomScrollView(
          controller: widget.controller,
          cacheExtent: widget.cacheExtent,
          physics: _physics,
          slivers: sliverList,
        ),
      ),
    );
  }
}
