import 'package:flutter/material.dart';

enum MillTagType {
  normal,
  warning,
  error,
}

class MillTag extends StatefulWidget {
  MillTag(
    String text, {
    Key key,
    this.showClose = true,
    this.onClose,
    this.color = const Color(0xfff0f0f0),
    this.textColor = const Color(0xff343434),
    this.height = 28,
    this.fontSize = 14,
    this.id,
    this.maxWidth = 160,
    this.type,
  })  : this.text = text,
        super(key: key);

  final MillTagType type;
  final String text;
  final Color color;
  final Color textColor;
  final double height;
  final double fontSize;
  final Function onClose;
  final bool showClose;
  final double maxWidth;
  final id;

  _MillTag createState() => _MillTag();
}

class _MillTag extends State<MillTag> {
  Map thems = {
    MillTagType.normal: {
      'color': Color(0xffecf5ff),
      'textColor': Color(0xff409eff),
    },
    MillTagType.warning: {
      'color': Color(0xfffdf6ec),
      'textColor': Color(0xffe6a23c),
    },
    MillTagType.error: {
      'color': Color(0xfffef0f0),
      'textColor': Color(0xfff56c6c),
    }
  };
  @override
  Widget build(BuildContext context) {
    Color color = widget.color;
    Color textColor = widget.textColor;
    if (widget.type != null) {
      color = thems[widget.type]['color'];
      textColor = thems[widget.type]['textColor'];
    }
    return Stack(
      children: [
        UnconstrainedBox(
          child: Container(
            height: widget.height,
            alignment: Alignment.centerLeft,
            decoration: BoxDecoration(
              color: color,
              borderRadius: BorderRadius.circular(widget.height / 2),
            ),
            padding: EdgeInsets.only(
              left: 12,
              right: widget.showClose ? 24 : 12,
            ),
            child: LimitedBox(
              maxWidth: widget.maxWidth,
              child: Text(
                widget.text,
                maxLines: 1,
                overflow: TextOverflow.ellipsis,
                style: TextStyle(
                  fontSize: widget.fontSize,
                  color: textColor,
                ),
              ),
            ),
          ),
        ),
        widget.showClose
            ? Positioned(
                top: 1,
                right: 0,
                child: GestureDetector(
                  onTap: () {
                    if (widget.onClose != null) {
                      widget.onClose(widget.id);
                    }
                  },
                  child: Container(
                    color: Colors.transparent,
                    padding: EdgeInsets.only(
                      right: 8,
                      left: 12,
                    ),
                    height: widget.height,
                    child: Icon(
                      Icons.close,
                      size: widget.fontSize,
                      color: textColor,
                    ),
                  ),
                ),
              )
            : Container(),
      ],
    );
  }
}
